@@ -1,7 +1,7 @@
 #!/usr/bin/env python
+
 from random import randint
 from time   import sleep
-from os     import listdir
 import subprocess
 import pty
 import socket
@@ -14,6 +14,24 @@ import pwd
 import stat
 import re
 
+#get testconfig
+# This assumes Makefile.in in main dir, but only Makefile in test dir.
+try:
+  sys.path += [os.getenv("PWD") + '/test', os.getenv('PWD')]
+  from autotest_config import *
+
+except ImportError:
+  print "\n*** Error importing autotest_config.py: "
+  sys.exit()
+
+if USE_TEST_SUITE == "no":
+  print "\n*** DMTCP test suite is disabled. To re-enable the test suite,\n" + \
+        "***  re-configure _without_ './configure --disable-test-suite'\n"
+  sys.exit()
+
+# Disable ptrace tests for now.
+PTRACE_SUPPORT="no"
+
 signal.alarm(1800)  # half hour
 
 if sys.version_info[0] != 2 or sys.version_info[0:2] < (2,4):
@@ -21,10 +39,21 @@ if sys.version_info[0] != 2 or sys.version_info[0:2] < (2,4):
   print "Change the beginning of test/autotest.py if you believe you can run."
   sys.exit(1)
 
-#get testconfig
+if sys.version_info[0] == 2 and sys.version_info[1] >= 7:
+  uname_m = subprocess.check_output(['uname', '-m'])
+  uname_p = subprocess.check_output(['uname', '-p'])
+else:
+  uname_m = subprocess.Popen(['uname', '-m'],
+                             stdout=subprocess.PIPE).communicate()[0]
+  uname_p = subprocess.Popen(['uname', '-p'],
+                             stdout=subprocess.PIPE).communicate()[0]
+
 # This assumes Makefile.in in main dir, but only Makefile in test dir.
 os.system("test -f Makefile || ./configure")
-import testconfig
+if USE_TEST_SUITE == "no":
+  print "\n*** DMTCP test suite is disabled. To re-enable the test suite,\n" + \
+          "***  re-configure _without_ './configure --disable-test-suite'\n"
+  sys.exit()
 
 #number of checkpoint/restart cycles
 CYCLES=2
@@ -34,24 +63,15 @@ RETRIES=2
 
 #Sleep after each program startup (sec)
 DEFAULT_S=0.3
-if sys.version_info[0] == 2 and sys.version_info[0:2] >= (2,7) and \
-    subprocess.check_output(['uname', '-p'])[0:3] == 'arm':
+if uname_p[0:3] == 'arm':
   DEFAULT_S *= 2
+#Allow extra time for slower CPUs
+if uname_m in ["i386", "i486", "i586", "i686", "armv7", "armv71"]:
+  DEFAULT_S *= 4
 
-if testconfig.MTCP_USE_PROC_MAPS == "yes":
-  DEFAULT_S = 2*DEFAULT_S
 S=DEFAULT_S
 #Appears as S*SLOW in code.  If --slow, then SLOW=5
 SLOW=1
-#In the case of gdb, even if both gdb and the inferior are running after
-#ckpt or restart, this does not guarantee that the ptrace-related work
-#(that is needed at resume or restart) is over. The ptrace related work happens
-#in the signal handler. Proceeding while still being inside the signal handler,
-#can lead to bad consquences. To play it on the safe side, PTRACE_SLEEP was
-#set at 2 seconds.  (Until this is fixed, --enable-ptrace-support will
-#remain experimental.)
-#if testconfig.PTRACE_SUPPORT == "yes":
-#  PTRACE_SLEEP=2
 
 #Max time to wait for ckpt/restart to finish (sec)
 TIMEOUT=10
@@ -70,15 +90,21 @@ BUFFER_SIZE=4096*8
 #False redirects process stderr
 VERBOSE=False
 
+#Should we retry on a failure?
+RETRY_ONCE=False
+
 #Run (most) tests with user default (usually with gzip enable)
 GZIP=os.getenv('DMTCP_GZIP') or "1"
 
-#Warn cant create a file of size:
+#Warn if can't create a file of size:
 REQUIRE_MB=50
 
 #Binaries
 BIN="./bin/"
 
+#Checkpoint command to send to coordinator
+CKPT_CMD='c'
+
 #parse program args
 args={}
 for i in sys.argv:
@@ -89,6 +115,16 @@ for i in sys.argv:
     CYCLES=100000
   if i=="--slow":
     SLOW=5
+  if i=="--retry-once":
+    RETRY_ONCE = True
+  #TODO:  Install SIGSEGV handler with infinite loop, and add to LD_PRELOAD
+  #In test/Makefile, build libcatchsigsegv.so
+  #Add --catchsigsegv  to usage string.
+  # if i=="--catchsigsegv":
+  #   if os.getenv('LD_PRELOAD'):
+  #     os.environ['LD_PRELOAD'] += ':libcatchsigsegv.so'
+  #   else:
+  #     os.environ['LD_PRELOAD'] = 'libcatchsigsegv.so'
   if i=="-h" or i=="--help":
     print ("USAGE "+sys.argv[0]+
       " [-v] [--stress] [--slow] [testname] [testname ...]")
@@ -124,10 +160,10 @@ def splitWithQuotes(string):
     if string[i] == "'" or string[i] == '"':
       # This triggers twice in:  '"..."'  (on first ' and second ")
       if xor(inSingleQuotes, inDoubleQuotes) and not isOuter: # if beg. of quote
-	isOuter = string[i]
+        isOuter = string[i]
         string = replaceChar(string, i, '#')
       elif isOuter == string[i]:  # if end of quote
-	isOuter = False
+        isOuter = False
         string = replaceChar(string, i, '#')
     if not inSingleQuotes and not inDoubleQuotes and string[i] == ' ':
       # FIXME (Is there any destructive way to do this?)
@@ -137,14 +173,19 @@ def splitWithQuotes(string):
 
 def shouldRunTest(name):
   # FIXME:  This is a hack.  We should have created var, testNaems and use here
-  if len(sys.argv) <= 1+(VERBOSE==True)+(SLOW!=1)+(CYCLES!=2):
+  if len(sys.argv) <= 1+(VERBOSE==True)+(SLOW!=1)+(CYCLES!=2)+(RETRY_ONCE==True):
     return True
   return name in sys.argv
 
 #make sure we are in svn root
-if os.system("test -d bin") is not 0:
+if os.system("test -d bin") != 0:
   os.chdir("..")
-assert os.system("test -d bin") is 0
+if USE_M32:
+  assert os.system("test -d bin") == 0, \
+  "  bin/dmtcp_launch not found.  Please configure and build the\n" + \
+  "  default 64-bit mode before configuring with --enable-m32 and re-building"
+else:
+  assert os.system("test -d bin") == 0
 
 #make sure dmtcp is built
 if os.system("make -s --no-print-directory tests") != 0:
@@ -154,7 +195,9 @@ if os.system("make -s --no-print-directory tests") != 0:
 #pad a string and print/flush it
 def printFixed(str, w=1):
   # The comma at end of print prevents a "newline", but still adds space.
-  print str.ljust(w),
+  if sys.version_info[0] == 2:
+    # Replace "print str.ljust(w),"  by version compatible w/ python3:
+    os.write(sys.stdout.fileno(), str.ljust(w).encode("ascii"))
   sys.stdout.flush()
 
 #exception on failed check
@@ -183,7 +226,7 @@ def launch(cmd):
   if VERBOSE:
     print "Launching... ", cmd
   cmd = splitWithQuotes(cmd);
-  # Example cmd:  dmtcp_checkpoint screen ...
+  # Example cmd:  dmtcp_launch screen ...
   ptyMode = False
   for str in cmd:
     # Checkpoint image can be emacs23_x, or whatever emacs is a link to.
@@ -229,8 +272,8 @@ def launch(cmd):
       childStderr = subprocess.STDOUT # Mix stderr into stdout file object
     # NOTE:  This might be replaced by shell=True in call to subprocess.Popen
     proc = subprocess.Popen(cmd, bufsize=BUFFER_SIZE,
-		 stdin=subprocess.PIPE, stdout=childStdout,
-		 stderr=childStderr, close_fds=True)
+                 stdin=subprocess.PIPE, stdout=childStdout,
+                 stderr=childStderr, close_fds=True)
   return proc
 
 #randomize port and dir, so multiple processes works
@@ -249,9 +292,29 @@ if not VERBOSE:
 if VERBOSE:
   print "coordinator port:  " + os.environ['DMTCP_PORT']
 
-#verify there is enough free space
+# We'll copy ckptdir to DMTCP_TMPDIR in case of error.
+def dmtcp_tmpdir():
+  tmpdir = os.getenv('DMTCP_TMPDIR') or os.getenv('TMPDIR') or '/tmp'
+  return tmpdir + '/dmtcp-' + os.environ['USER'] + '@' + socket.gethostname()
+
+def free_diskspace(dir):
+  s = os.statvfs('.')
+  return s.f_bavail * s.f_frsize
+
+# We'll save core dumps in our default directory (usually dmtcp-autotest-*)
+# We can use the lesser of half the free disk space of filesystem or 100 MB.
+if free_diskspace(ckptDir) > 20*1024*1024:
+  oldLimit = resource.getrlimit(resource.RLIMIT_CORE)
+  newLimit = [min(free_diskspace(ckptDir)/2, 100*1024*1024), oldLimit[1]]
+  if oldLimit[1] != resource.RLIM_INFINITY:  # Keep soft limit below hard limit
+    newLimit[0] = min(newLimit[0], oldLimit[1])
+  resource.setrlimit(resource.RLIMIT_CORE, newLimit)
+
+# This can be slow.
+print("Verifying there is enough disk space ...")
 tmpfile=ckptDir + "/freeSpaceTest.tmp"
-if os.system("dd if=/dev/zero of="+tmpfile+" bs=1MB count="+str(REQUIRE_MB)+" 2>/dev/null") != 0:
+if os.system("dd if=/dev/zero of=" + tmpfile + " bs=1MB count=" +
+             str(REQUIRE_MB) + " 2>/dev/null") != 0:
   GZIP="1"
   print '''
 
@@ -264,16 +327,10 @@ Many of the tests below may fail due to insufficient space.
 os.system("rm -f "+tmpfile)
 
 os.environ['DMTCP_GZIP'] = GZIP
-
-# Temporary hack until DMTCP cleans up when using --enable-ptrace-support
-def deletePtraceFiles():
-  tmpdir = os.getenv("TMPDIR", "/tmp")  # if "TMPDIR" not set, return "/tmp"
-  tmpdir += "/dmtcp-" + pwd.getpwuid(os.getuid()).pw_name + \
-            "@" + socket.gethostname()
-  os.system("cd "+tmpdir+"; "+
-	    "rm -f ptrace_shared.txt ptrace_setoptions.txt \
-	     ptrace_ckpthreads.txt ptrace_shared.txt ptrace_setoptions.txt \
-	     ptrace_ckpthreads.txt new_ptrace_shared.txt ckpt_leader_file.txt")
+if os.getenv('LD_LIBRARY_PATH'):
+    os.environ['LD_LIBRARY_PATH'] += ':' + os.getenv("PWD")+"/lib"
+else:
+    os.environ['LD_LIBRARY_PATH'] = os.getenv("PWD")+"/lib"
 
 #launch the coordinator
 coordinator = launch(BIN+"dmtcp_coordinator")
@@ -286,7 +343,8 @@ def coordinatorCmd(cmd):
     coordinator.stdin.write(cmd+"\n")
     coordinator.stdin.flush()
   except:
-    raise CheckFailed("failed to write '%s' to coordinator (pid: %d)" %  (cmd, coordinator.pid))
+    raise CheckFailed("failed to write '%s' to coordinator (pid: %d)" % \
+                      (cmd, coordinator.pid))
 
 #clean up after ourselves
 def SHUTDOWN():
@@ -317,19 +375,29 @@ def WAITFOR(test, msg):
 def getStatus():
   coordinatorCmd('s')
 
-  if coordinator.poll() >= 0:
+  returncode = coordinator.poll()
+  if returncode:
+    if returncode < 0:
+      print "Coordinator terminated by signal ", str(-returncode)
     CHECK(False, "coordinator died unexpectedly")
     return (-1, False)
 
   while True:
     try:
-      line=coordinator.stdout.readline().strip()
+      line=coordinator.stdout.readline()
+      if not line:  # Immediate empty string on stdout means EOF
+        CHECK(False, "coordinator died unexpectedly")
+        return (-1, False)
+      line=line.strip()
       if line=="Status...":
         break;
       if VERBOSE:
         print "Ignoring line from coordinator: ", line
+        sleep(1)
     except IOError, (errno, strerror):
-      if coordinator.poll() >= 0:
+      if coordinator.poll():
+        if coordinator.poll() < 0:
+          print "Coordinator terminated by signal ", str(-returncode)
         CHECK(False, "coordinator died unexpectedly")
         return (-1, False)
       if errno==4: #Interrupted system call
@@ -359,24 +427,25 @@ def clearCkptDir():
           #   os.remove(os.path.join(root, name))
           os.remove(os.path.join(root, name))
         except OSError, e:
-	  if e.errno != errno.ENOENT:  # Maybe ckpt_*_dmtcp.temp was renamed.
-	    raise e
+          if e.errno != errno.ENOENT:  # Maybe ckpt_*_dmtcp.temp was renamed.
+            raise e
       for name in dirs:
         os.rmdir(os.path.join(root, name))
 
 def getNumCkptFiles(dir):
-  return len(filter(lambda f: f.startswith("ckpt_") and f.endswith(".dmtcp"), listdir(dir)))
+  return len(filter(lambda f: f.startswith("ckpt_") and f.endswith(".dmtcp"),\
+                              os.listdir(dir)))
 
 
 # Test a given list of commands to see if they checkpoint
 # runTest() sets up a keyboard interrupt handler, and then calls this function.
 def runTestRaw(name, numProcs, cmds):
   #the expected/correct running status
-  if testconfig.USE_M32 == "1":
-    def forall(fnc, lst):
-      return reduce(lambda x, y: x and y, map(fnc, lst))
-    if not forall(lambda x: x.startswith("./test/"), cmds):
-      return
+#  if USE_M32:
+#    def forall(fnc, lst):
+#      return reduce(lambda x, y: x and y, map(fnc, lst))
+#    if not forall(lambda x: x.startswith("./test/"), cmds):
+#      return
   status=(numProcs, True)
   procs=[]
 
@@ -399,7 +468,7 @@ def runTestRaw(name, numProcs, cmds):
     coordinatorCmd('k')
     try:
       WAITFOR(lambda: getStatus()==(0, False),
-	      lambda:"coordinator kill command failed")
+              lambda:"coordinator kill command failed")
     except CheckFailed:
       global coordinator
       coordinatorCmd('q')
@@ -426,21 +495,21 @@ def runTestRaw(name, numProcs, cmds):
       try:
         os.waitpid(x.pid, os.WNOHANG)
       except OSError, e:
-	if e.errno != errno.ECHILD:
-	  raise e
+        if e.errno != errno.ECHILD:
+          raise e
       procs.remove(x)
 
   def testCheckpoint():
     #start checkpoint
-    coordinatorCmd('c')
+    coordinatorCmd(CKPT_CMD)
 
     #wait for files to appear and status to return to original
     WAITFOR(lambda: getNumCkptFiles(ckptDir)>0 and \
-                    doesStatusSatisfy(getStatus(), status),
+                    (CKPT_CMD == 'xc' or doesStatusSatisfy(getStatus(), status)),
             wfMsg("checkpoint error"))
 
     #make sure the right files are there
-    numFiles=getNumCkptFiles(ckptDir) # len(listdir(ckptDir))
+    numFiles=getNumCkptFiles(ckptDir) # len(os.listdir(ckptDir))
     CHECK(doesStatusSatisfy((numFiles,True),status),
           "unexpected number of checkpoint files, %s procs, %d files"
           % (str(status[0]), numFiles))
@@ -448,14 +517,14 @@ def runTestRaw(name, numProcs, cmds):
   def testRestart():
     #build restart command
     cmd=BIN+"dmtcp_restart --quiet"
-    for i in listdir(ckptDir):
+    for i in os.listdir(ckptDir):
       if i.endswith(".dmtcp"):
         cmd+= " "+ckptDir+"/"+i
     #run restart and test if it worked
     procs.append(launch(cmd))
     WAITFOR(lambda: doesStatusSatisfy(getStatus(), status),
             wfMsg("restart error"))
-    if testconfig.HBICT_DELTACOMP == "no":
+    if HBICT_DELTACOMP == "no":
       clearCkptDir()
 
   try:
@@ -470,7 +539,7 @@ def runTestRaw(name, numProcs, cmds):
 
     #start user programs
     for cmd in cmds:
-      procs.append(launch(BIN+"dmtcp_checkpoint "+cmd))
+      procs.append(launch(BIN+"dmtcp_launch "+cmd))
       sleep(S*SLOW)
 
     WAITFOR(lambda: doesStatusSatisfy(getStatus(), status),
@@ -485,8 +554,6 @@ def runTestRaw(name, numProcs, cmds):
       #  of this function:  testRestart
       testCheckpoint()
       printFixed("PASSED ")
-      #if testconfig.PTRACE_SUPPORT == "yes":
-      #  sleep(PTRACE_SLEEP)
       testKill()
 
       printFixed("rstr:")
@@ -494,17 +561,39 @@ def runTestRaw(name, numProcs, cmds):
         try:
           testRestart()
           printFixed("PASSED")
-          #if testconfig.PTRACE_SUPPORT == "yes":
-          #  sleep(PTRACE_SLEEP)
           break
         except CheckFailed, e:
           if j == RETRIES-1:
+            # Save checkpoint images for later diagnosis.
+            if os.path.isdir(dmtcp_tmpdir()) and os.path.isdir(ckptDir):
+              if subprocess.call( ("cp -pr " + ckptDir + ' '
+                                   + dmtcp_tmpdir()).split() ) == 0:
+                print "\n***** Copied checkpoint images to " + dmtcp_tmpdir() \
+                      + "/" + ckptDir
             raise e
           else:
-            printFixed("FAILED retry:")
+            printFixed("FAILED ")
+            (oldpid, oldstatus) = os.waitpid(procs[-1].pid, os.WNOHANG)
+            if oldpid == procs[-1].pid:
+              if os.WIFEXITED(oldstatus):
+                printFixed("(first process exited: oldstatus "
+                           + str(os.WEXITSTATUS(oldstatus)) + ")")
+              if os.WIFSIGNALED(oldstatus):
+                printFixed("(first process rec'd signal "
+                           + str(os.WTERMSIG(oldstatus)) + ")")
+              if os.WCOREDUMP(oldstatus):
+                coredump = "core." + str(oldpid)
+                if os.path.isdir(dmtcp_tmpdir()) and os.path.isfile(coredump):
+                  if subprocess.call( ("cp -pr " + coredump + ' '
+                                   + dmtcp_tmpdir()).split() ) == 0:
+                    printFixed(" (" + coredump + " copied to DMTCP_TMPDIR:" +
+                               dmtcp_tmpdir() + "/)")
+            else:
+              printFixed("(first process didn't die)")
+            printFixed(" retry:")
             testKill()
       if i != CYCLES - 1:
-	printFixed(";")
+        printFixed("; ")
 
     testKill()
     print #newline
@@ -521,6 +610,9 @@ def runTestRaw(name, numProcs, cmds):
       SHUTDOWN()
       saveResultsNMI()
       sys.exit(1)
+    if RETRY_ONCE:
+      clearCkptDir()
+      raise e
 
   clearCkptDir()
 
@@ -532,17 +624,25 @@ def getProcessChildren(pid):
 
 # If the user types ^C, then kill all child processes.
 def runTest(name, numProcs, cmds):
-  try:
-    runTestRaw(name, numProcs, cmds)
-  except KeyboardInterrupt:
-    for pid in getProcessChildren(os.getpid()):
-      try:
-        os.kill(pid, signal.SIGKILL)
-      except OSError: # This happens if pid already died.
-        pass
+  for i in range(2):
+    try:
+      runTestRaw(name, numProcs, cmds)
+      break;
+    except KeyboardInterrupt:
+      for pid in getProcessChildren(os.getpid()):
+        try:
+          os.kill(pid, signal.SIGKILL)
+        except OSError: # This happens if pid already died.
+          pass
+    except CheckFailed, e:
+      if not RETRY_ONCE:
+        break
+      if i == 0:
+        stats[1]-=1
+        print "Trying once again"
 
 def saveResultsNMI():
-  if testconfig.DEBUG == "yes":
+  if DEBUG == "yes":
     # WARNING:  This can cause a several second delay on some systems.
     host = socket.getfqdn()
     if re.search("^nmi-.*.cs.wisc.edu$", host) or \
@@ -551,15 +651,15 @@ def saveResultsNMI():
       target = "./dmtcp-" + pwd.getpwuid(os.getuid()).pw_name + \
                "@" + socket.gethostname()
       cmd = "mkdir results; cp -pr " + tmpdir + "/" + target + \
-	       " ./dmtcp/src/dmtcphijack.so" + \
-	       " ./dmtcp/src/dmtcp_coordinator" + \
+               " ./dmtcp/src/libdmtcp.so" + \
+               " ./dmtcp/src/dmtcp_coordinator" + \
                " ./mtcp/libmtcp.so" + \
                " results/"
       os.system(cmd)
       cmd = "tar zcf ../results.tar.gz ./results; rm -rf results"
       os.system(cmd)
       print "\n*** results.tar.gz ("+tmpdir+"/"+target+ \
-					      ") written to DMTCP_ROOT/.. ***"
+                                              ") written to DMTCP_ROOT/.. ***"
 
 print "== Tests =="
 
@@ -576,13 +676,7 @@ runTest("dmtcp1",        1, ["./test/dmtcp1"])
 
 runTest("dmtcp2",        1, ["./test/dmtcp2"])
 
-# dmtcp3 creates 10 threads; Keep checkpoint image small by using gzip
-# Also, it needs some extra time to startup
-S=2
-os.environ['DMTCP_GZIP'] = "1"
 runTest("dmtcp3",        1, ["./test/dmtcp3"])
-os.environ['DMTCP_GZIP'] = GZIP
-S=DEFAULT_S
 
 runTest("dmtcp4",        1, ["./test/dmtcp4"])
 
@@ -591,7 +685,7 @@ runTest("dmtcp4",        1, ["./test/dmtcp4"])
 # in low memory (0x110000) in the statically linked mtcp_restart executable.
 oldLimit = resource.getrlimit(resource.RLIMIT_STACK)
 # oldLimit[1] is old hard limit
-if oldLimit[1] == -1L:
+if oldLimit[1] == resource.RLIM_INFINITY:
   newCurrLimit = 8L*1024*1024
 else:
   newCurrLimit = min(8L*1024*1024, oldLimit[1])
@@ -599,22 +693,41 @@ resource.setrlimit(resource.RLIMIT_STACK, [newCurrLimit, oldLimit[1]])
 runTest("dmtcp5",        2, ["./test/dmtcp5"])
 resource.setrlimit(resource.RLIMIT_STACK, oldLimit)
 
+# Test for a bunch of system calls. We want to use the 'xc' mode for
+# checkpointing so that the process is killed right after checkpoint. Otherwise
+# the syscall-tester could fail in the following case:
+#   1. create and open temp file
+#   2. close temp file
+#   3. ckpt
+#   4. unlink temp file
+# If the last step is executed before the process is killed after ckpt-resume,
+# the file would have been deleted from the disk. However, on restart, the test
+# program will try to unlink the file once again, but the unlink operation will
+# fail, causing the test to fail.
+old_ckpt_cmd = CKPT_CMD
+CKPT_CMD = 'xc'
+runTest("syscall-tester",  1, ["./test/syscall-tester"])
+CKPT_CMD = old_ckpt_cmd
+
+# Test for files opened with WRONLY mode and later unlinked.
+runTest("file1",         1, ["./test/file1"])
+
 runTest("dmtcpaware1",   1, ["./test/dmtcpaware1"])
 
 PWD=os.getcwd()
 runTest("plugin-sleep2", 1, ["--with-plugin "+
-			     PWD+"/test/plugin/sleep1/dmtcp_sleep1hijack.so:"+
-			     PWD+"/test/plugin/sleep2/dmtcp_sleep2hijack.so "+
-			     "./test/dmtcp1"])
+                             PWD+"/test/plugin/sleep1/dmtcp_sleep1hijack.so:"+
+                             PWD+"/test/plugin/sleep2/dmtcp_sleep2hijack.so "+
+                             "./test/dmtcp1"])
 
 runTest("plugin-example-db", 2, ["--with-plugin "+
-			    PWD+"/test/plugin/example-db/dmtcp_example-dbhijack.so "+
-			     "env EXAMPLE_DB_KEY=1 EXAMPLE_DB_KEY_OTHER=2 "+
-			     "./test/dmtcp1",
-			         "--with-plugin "+
-			    PWD+"/test/plugin/example-db/dmtcp_example-dbhijack.so "+
-			     "env EXAMPLE_DB_KEY=2 EXAMPLE_DB_KEY_OTHER=1 "+
-			     "./test/dmtcp1"])
+                            PWD+"/test/plugin/example-db/dmtcp_example-dbhijack.so "+
+                             "env EXAMPLE_DB_KEY=1 EXAMPLE_DB_KEY_OTHER=2 "+
+                             "./test/dmtcp1",
+                                 "--with-plugin "+
+                            PWD+"/test/plugin/example-db/dmtcp_example-dbhijack.so "+
+                             "env EXAMPLE_DB_KEY=2 EXAMPLE_DB_KEY_OTHER=1 "+
+                             "./test/dmtcp1"])
 
 # Test special case:  gettimeofday can be handled within VDSO segment.
 runTest("gettimeofday",  1, ["./test/gettimeofday"])
@@ -628,12 +741,21 @@ runTest("stale-fd",      2, ["./test/stale-fd"])
 # Disable procfd1 until we fix readlink
 #runTest("procfd1",       2, ["./test/procfd1"])
 
+# popen1 can have more than one processes
+#runTest("popen1",          1, ["./test/popen1"])
+
 runTest("poll",          1, ["./test/poll"])
 
 runTest("forkexec",      2, ["./test/forkexec"])
 
-if testconfig.PID_VIRTUALIZATION == "yes":
-  runTest("waitpid",      2, ["./test/waitpid"])
+if HAS_SSH == "yes":
+  S=5*DEFAULT_S
+  runTest("sshtest",     4, ["./test/sshtest"])
+  S=DEFAULT_S
+
+S=2*DEFAULT_S
+runTest("waitpid",      2, ["./test/waitpid"])
+S=DEFAULT_S
 
 runTest("client-server", 2, ["./test/client-server"])
 
@@ -646,18 +768,37 @@ os.environ['DMTCP_GZIP'] = "0"
 
 runTest("shared-memory", 2, ["./test/shared-memory"])
 
-# This is arguably a bug in the Linux kernel 3.2 for ARM.
-if sys.version_info[0] == 2 and sys.version_info[0:2] >= (2,7) and \
-    subprocess.check_output(['uname', '-p'])[0:3] == 'arm':
-  print "On ARM, there is a known issue with the sysv-shm test. Not running it."
-runTest("sysv-shm",      2, ["./test/sysv-shm"])
+runTest("sysv-shm1",     2, ["./test/sysv-shm1"])
+runTest("sysv-shm2",     2, ["./test/sysv-shm2"])
+runTest("sysv-sem",      2, ["./test/sysv-sem"])
+runTest("sysv-msg",      2, ["./test/sysv-msg"])
+
+# ARM glibc 2.16 with Linux kernel 3.0 doesn't support mq_send, etc.
+if uname_p[0:3] == 'arm':
+  print "Skipping posix-mq1/mq2 tests; ARM/glibc/Linux does not support mq_send"
+elif TEST_POSIX_MQ == "yes":
+  runTest("posix-mq1",     2, ["./test/posix-mq1"])
+  runTest("posix-mq2",     2, ["./test/posix-mq2"])
 
 #Invoke this test when we drain/restore data in pty at checkpoint time.
-# runTest("pty",   2, ["./test/pty"])
+# runTest("pty1",   2, ["./test/pty1"])
+runTest("pty2",   2, ["./test/pty2"])
+
+#Invoke this test when support for timers is added to DMTCP.
+runTest("timer",   1, ["./test/timer"])
+runTest("clock",   1, ["./test/clock"])
 
 old_ld_library_path = os.getenv("LD_LIBRARY_PATH")
-os.environ['LD_LIBRARY_PATH'] = os.getenv("PWD")+"/test:"+os.getenv("PWD")
-runTest("dlopen",        1, ["./test/dlopen"])
+if old_ld_library_path:
+  os.environ['LD_LIBRARY_PATH'] += ':' + os.getenv("PWD") + \
+                                   "/test:" + os.getenv("PWD")
+else:
+  os.environ['LD_LIBRARY_PATH'] = os.getenv("PWD") + "/test:" + os.getenv("PWD")
+runTest("dlopen1",        1, ["./test/dlopen1"])
+# Disable the dlopen2 test until we can figure out a way to handle calls to
+# fork/exec/wait during library intialization with dlopen().
+#if not USE_M32:
+#  runTest("dlopen2",        1, ["./test/dlopen2"])
 if old_ld_library_path:
   os.environ['LD_LIBRARY_PATH'] = old_ld_library_path
 else:
@@ -665,79 +806,91 @@ else:
 
 runTest("pthread1",      1, ["./test/pthread1"])
 runTest("pthread2",      1, ["./test/pthread2"])
-S=3
+S=10*DEFAULT_S
 runTest("pthread3",      1, ["./test/pthread2 80"])
 S=DEFAULT_S
-runTest("pthread4",      1, ["./test/pthread4 80"])
+runTest("pthread4",      1, ["./test/pthread4"])
+runTest("pthread5",      1, ["./test/pthread5"])
 
 os.environ['DMTCP_GZIP'] = "1"
 runTest("gzip",          1, ["./test/dmtcp1"])
 os.environ['DMTCP_GZIP'] = GZIP
 
-if testconfig.HAS_READLINE == "yes":
+if HAS_READLINE == "yes":
   runTest("readline",    1,  ["./test/readline"])
 
 runTest("perl",          1, ["/usr/bin/perl"])
 
-if testconfig.HAS_PYTHON == "yes":
+if HAS_PYTHON == "yes":
   runTest("python",      1, ["/usr/bin/python"])
 
-if testconfig.PID_VIRTUALIZATION == "yes":
-  os.environ['DMTCP_GZIP'] = "0"
-  runTest("bash",        2, ["/bin/bash --norc -c 'ls; sleep 30; ls'"])
-  os.environ['DMTCP_GZIP'] = GZIP
+os.environ['DMTCP_GZIP'] = "0"
+runTest("bash",        2, ["/bin/bash --norc -c 'ls; sleep 30; ls'"])
+os.environ['DMTCP_GZIP'] = GZIP
 
-if testconfig.HAS_DASH == "yes":
+if HAS_DASH == "yes":
   os.environ['DMTCP_GZIP'] = "0"
   os.unsetenv('ENV')  # Delete reference to dash initialization file
   runTest("dash",        2, ["/bin/dash -c 'ls; sleep 30; ls'"])
   os.environ['DMTCP_GZIP'] = GZIP
 
-if testconfig.HAS_TCSH == "yes":
+if HAS_TCSH == "yes":
   os.environ['DMTCP_GZIP'] = "0"
   runTest("tcsh",        2, ["/bin/tcsh -f -c 'ls; sleep 30; ls'"])
   os.environ['DMTCP_GZIP'] = GZIP
 
-if testconfig.HAS_ZSH == "yes":
+if HAS_ZSH == "yes":
   os.environ['DMTCP_GZIP'] = "0"
-  S=1
+  S=3*DEFAULT_S
   runTest("zsh",         2, ["/bin/zsh -f -c 'ls; sleep 30; ls'"])
   S=DEFAULT_S
   os.environ['DMTCP_GZIP'] = GZIP
 
-if testconfig.HAS_VIM == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
+if HAS_VIM == "yes":
   # Wait to checkpoint until vim finishes reading its initialization files
-  S=3
+  S=10*DEFAULT_S
   if sys.version_info[0:2] >= (2,6):
     # Delete previous vim processes.  Vim behaves poorly with stale processes.
-    vimCommand = testconfig.VIM + " /etc/passwd +3" # +3 makes cmd line unique
+    vimCommand = VIM + " /etc/passwd +3" # +3 makes cmd line unique
     def killCommand(cmdToKill):
-      if os.getenv('USER') == None:
+      if os.getenv('USER') == None or HAS_PS == 'no':
         return
-      ps = subprocess.Popen(['ps', '-u', os.environ['USER'], '-o', 'pid,command'],
-    		            stdout=subprocess.PIPE).communicate()[0]
+      ps = subprocess.Popen(['ps', '-u', os.environ['USER'], '-o',
+                             'pid,command'],
+                            stdout=subprocess.PIPE).communicate()[0]
       for row in ps.split('\n')[1:]:
         cmd = row.split(None, 1) # maxsplit=1
-        if cmd and cmd[1] == cmdToKill:
+        if len(cmd) > 1 and cmd[1] == cmdToKill:
           os.kill(int(cmd[0]), signal.SIGKILL)
     killCommand(vimCommand)
     runTest("vim",       1,  ["env TERM=vt100 " + vimCommand])
     killCommand(vimCommand)
   S=DEFAULT_S
 
-if testconfig.HAS_EMACS == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
-  # Wait to checkpoint until emacs finishes reading its initialization files
-  S=4
-  if sys.version_info[0:2] >= (2,6):
+if sys.version_info[0:2] >= (2,6):
+  #On some systems, "emacs -nw" launches dbus-daemon processes in
+  #background throwing off the number of processes in the computation. The
+  #test thus fails. The fix is to launch emacs-nox, if found. emacs-nox
+  #doesn't launch any background processes.
+  S=15*DEFAULT_S
+  if HAS_EMACS_NOX == "yes":
+    # Wait to checkpoint until emacs finishes reading its initialization files
+    # Under emacs23, it opens /dev/tty directly in a new fd.
+    # To avoid this, consider using emacs --batch -l EMACS-LISTP-CODE ...
+    # ... or else a better pty wrapper to capture emacs output to /dev/tty.
+    runTest("emacs",     1,  ["env TERM=vt100 /usr/bin/emacs-nox" +
+                              " --no-init-file /etc/passwd"])
+  elif HAS_EMACS == "yes":
+    # Wait to checkpoint until emacs finishes reading its initialization files
     # Under emacs23, it opens /dev/tty directly in a new fd.
     # To avoid this, consider using emacs --batch -l EMACS-LISTP-CODE ...
     # ... or else a better pty wrapper to capture emacs output to /dev/tty.
     runTest("emacs",     1,  ["env TERM=vt100 /usr/bin/emacs -nw" +
-                              " --no-init-file /etc/passwd"])
+                                " --no-init-file /etc/passwd"])
   S=DEFAULT_S
 
-if testconfig.HAS_SCRIPT == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
-  S=2
+if HAS_SCRIPT == "yes":
+  S=7*DEFAULT_S
   if sys.version_info[0:2] >= (2,6):
     # NOTE: If 'script' fails, try raising value of S, above, to larger number.
     #  Arguably, there is a bug in glibc, in that locale-archive can be 100 MB.
@@ -746,107 +899,129 @@ if testconfig.HAS_SCRIPT == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
     #  only 8KB of content in ASCII.  The 100 MB of locale-archive condenses
     #  to 25 MB _per process_ under gzip, but this can be slow at ckpt time.
     runTest("script",    4,  ["/usr/bin/script -f" +
-    			      " -c 'bash -c \"ls; sleep 30\"'" +
-    			      " dmtcp-test-typescript.tmp"])
+                              " -c 'bash -c \"ls; sleep 30\"'" +
+                              " dmtcp-test-typescript.tmp"])
   os.system("rm -f dmtcp-test-typescript.tmp")
   S=DEFAULT_S
 
 # SHOULD HAVE screen RUN SOMETHING LIKE:  bash -c ./test/dmtcp1
-if testconfig.HAS_SCREEN == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
-  S=1
+# FIXME: Currently fails on dekaksi due to DMTCP not honoring
+#        "Async-signal-safe functions" in signal handlers (see man 7 signal)
+if HAS_SCREEN == "yes":
+  S=3*DEFAULT_S
   if sys.version_info[0:2] >= (2,6):
-    runTest("screen",    3,  ["env TERM=vt100 " + testconfig.SCREEN +
+    runTest("screen",    3,  ["env TERM=vt100 " + SCREEN +
                                 " -c /dev/null -s /bin/sh"])
   S=DEFAULT_S
 
-if testconfig.PTRACE_SUPPORT == "yes" and \
-   (testconfig.HAS_STRACE == "yes" or testconfig.HAS_GDB == "yes"):
-  print "  Deleting files in /tmp/dmtcp-USER@HOST before ptrace tests.  (Until"
-  print "  this is fixed, --enable-ptrace-support will remain experimental.)"
-  deletePtraceFiles()
-  if testconfig.HAS_STRACE == "yes" and testconfig.PTRACE_SUPPORT == "yes":
-    S=1
-    if sys.version_info[0:2] >= (2,6):
-      runTest("strace",    2,  ["strace test/dmtcp2"])
+if PTRACE_SUPPORT == "yes" and ARM_HOST == "no" and \
+   sys.version_info[0:2] >= (2,6):
+  if HAS_STRACE == "yes":
+    S=10*DEFAULT_S
+    runTest("strace",    2,  ["--ptrace strace test/dmtcp2"])
     S=DEFAULT_S
 
-  deletePtraceFiles()
-  if testconfig.HAS_GDB == "yes" and testconfig.PTRACE_SUPPORT == "yes":
-    os.system("echo 'run' > dmtcp-gdbinit.tmp")
-    S=2
-    if sys.version_info[0:2] >= (2,6):
-      runTest("gdb",       2,  ["gdb -n -batch -x dmtcp-gdbinit.tmp test/dmtcp1"])
-    S=DEFAULT_S
-    os.system("rm -f dmtcp-gdbinit.tmp")
+  if HAS_GDB == "yes":
+    if uname_p[0:3] == 'arm':
+      print "On ARM, there is a known issue with DMTCP for gdb-* test." + \
+            "  Not running it."
+    else:
+      os.system("echo 'run' > dmtcp-gdbinit.tmp")
+      S=10*DEFAULT_S
+      runTest("gdb",          2,
+              ["--ptrace gdb -n -batch -x dmtcp-gdbinit.tmp test/dmtcp1"])
 
-  deletePtraceFiles()
-  if testconfig.HAS_GDB == "yes" and testconfig.PTRACE_SUPPORT == "yes":
-    os.system("echo 'run' > dmtcp-gdbinit.tmp")
-    S=2
-    if sys.version_info[0:2] >= (2,6):
-      runTest("gdb-pthread",2,  ["gdb -n -batch -x dmtcp-gdbinit.tmp test/dmtcp3"])
-    S=DEFAULT_S
-    os.system("rm -f dmtcp-gdbinit.tmp")
+      runTest("gdb-pthread0", 2,
+              ["--ptrace gdb -n -batch -x dmtcp-gdbinit.tmp test/dmtcp3"])
 
-if testconfig.HAS_JAVAC == "yes" and testconfig.HAS_JAVA == "yes":
-  S=3
+      # These tests currently fail sometimes (if the computation is checkpointed
+      # while a thread is being created). Re-enable them when this issue has
+      # been fixed in the ptrace plugin.
+      #runTest("gdb-pthread1", 2, ["gdb -n -batch -x dmtcp-gdbinit.tmp test/pthread1"])
+      #runTest("gdb-pthread2",2, ["gdb -n -batch -x dmtcp-gdbinit.tmp test/pthread2"])
+
+      S=DEFAULT_S
+      os.system("rm -f dmtcp-gdbinit.tmp")
+
+if HAS_JAVAC == "yes" and HAS_JAVA == "yes":
+  S=10*DEFAULT_S
   os.environ['CLASSPATH'] = './test'
-  if testconfig.HAS_SUN_ORACLE_JAVA == "yes":
-    runTest("java1",         1,  ["java -Xmx512M java1"])
-  else:
-    runTest("java1",         1,  ["java java1"])
+  runTest("java1",         1,  ["java -Xmx5M java1"])
   del os.environ['CLASSPATH']
   S=DEFAULT_S
 
-if testconfig.HAS_CILK == "yes":
+if HAS_CILK == "yes":
   runTest("cilk1",        1,  ["./test/cilk1 38"])
 
 # SHOULD HAVE gcl RUN LARGE FACTORIAL OR SOMETHING.
-if testconfig.HAS_GCL == "yes":
-  S=1
-  runTest("gcl",         1,  [testconfig.GCL])
+if HAS_GCL == "yes":
+  S=3*DEFAULT_S
+  runTest("gcl",         1,  [GCL])
   S=DEFAULT_S
 
-if testconfig.HAS_OPENMP == "yes":
+if HAS_OPENMP == "yes":
   runTest("openmp-1",         1,  ["./test/openmp-1"])
   runTest("openmp-2",         1,  ["./test/openmp-2"])
 
 # SHOULD HAVE matlab RUN LARGE FACTORIAL OR SOMETHING.
-if testconfig.HAS_MATLAB == "yes":
-  S=3
-  if sys.version_info[0:2] >= (2,6):
-    runTest("matlab-nodisplay", 1,  [testconfig.MATLAB+" -nodisplay -nojvm"])
+if HAS_MATLAB == "yes" and sys.version_info[0:2] >= (2,6):
+  S=10*DEFAULT_S
+  runTest("matlab-nodisplay", 1,  [MATLAB+" -nodisplay -nojvm"])
   S=DEFAULT_S
 
-if testconfig.HAS_MPICH == "yes":
-  runTest("mpd",         1, [testconfig.MPICH_MPD])
+if HAS_MPICH == "yes":
+  runTest("mpd",         1, [MPICH_MPD])
 
-  runTest("hellompich-n1", 4, [testconfig.MPICH_MPD,
-                           testconfig.MPICH_MPIEXEC+" -n 1 ./test/hellompich"])
+  runTest("hellompich-n1", 4,
+          [MPICH_MPD, MPICH_MPIEXEC + " -n 1 ./test/hellompich"])
 
-  runTest("hellompich-n2", 6, [testconfig.MPICH_MPD,
-                           testconfig.MPICH_MPIEXEC+" -n 2 ./test/hellompich"])
+  runTest("hellompich-n2", 6,
+          [MPICH_MPD, MPICH_MPIEXEC + " -n 2 ./test/hellompich"])
 
-  runTest("mpdboot",     1, [testconfig.MPICH_MPDBOOT+" -n 1"])
+  runTest("mpdboot",     1, [MPICH_MPDBOOT+" -n 1"])
 
-  #os.system(testconfig.MPICH_MPDCLEANUP)
+  #os.system(MPICH_MPDCLEANUP)
 
-# Temporarily disabling OpenMPI test as it fails on some distros (OpenSUSE 11.4)
-if testconfig.HAS_OPENMPI == "yes":
-  numProcesses = 5 + int(testconfig.USES_OPENMPI_ORTED == "yes")
+
+if HAS_OPENMPI == "yes":
+  # Compute:  USES_OPENMPI_ORTED
+  if os.path.isfile('./test/openmpi') and \
+     0 == os.system(OPENMPI_MPICC +
+                    " -o ./test_openmpi test/hellompi.c 2>/dev/null 1>&2"):
+    os.system("rm -f ./uses_openmpi_orted")
+    # The 'sleep 1' below may not fix the race, creating a runaway test_openmpi.
+    os.system('/bin/sh -c "$OPENMPI_MPIRUN -np 2 ./test_openmpi' +
+              '   2>/dev/null 1>&2 &'
+              ' sleep 1 &&'
+              ' ps auxw | grep $USER | grep -v grep | grep -q orted &&'
+              ' touch ./uses_openmpi_orted" 2>/dev/null')
+    os.system("/bin/kill -9 `ps -eo pid,args | grep test_openmpi |" +
+              " sed -e 's%\([0-9]\) .*$%\1%'` 2>/dev/null")
+    if os.path.exists('./uses_openmpi_orted'):
+      os.system('rm -f ./uses_openmpi_orted')
+      USES_OPENMPI_ORTED = "yes"
+    else:
+      USES_OPENMPI_ORTED = "no"
+  else:
+    HAS_OPENMPI = "no"
+  os.system('rm -f ./test_openmpi')
+
+#Temporarily disabling Open MPI test as it fails on some distros (OpenSUSE 11.4)
+if HAS_OPENMPI == "yes":
+  numProcesses = 5 + int(USES_OPENMPI_ORTED == "yes")
   # FIXME: Replace "[5,6]" by numProcesses when bug in configure is fixed.
   # /usr/bin/openmpi does not work if /usr/bin is not also in user's PATH
   oldPath = ""
   if not os.environ.has_key('PATH'):
     oldPath = None
-    os.environ['PATH'] = os.path.dirname(testconfig.OPENMPI_MPIRUN)
-  elif (not re.search(os.path.dirname(testconfig.OPENMPI_MPIRUN),
+    os.environ['PATH'] = os.path.dirname(OPENMPI_MPIRUN)
+  elif (not re.search(os.path.dirname(OPENMPI_MPIRUN),
                      os.environ['PATH'])):
     oldPath = os.environ['PATH']
-    os.environ += ":" + os.path.dirname(testconfig.OPENMPI_MPIRUN)
-  S=1
-  runTest("openmpi", [5,6], [testconfig.OPENMPI_MPIRUN + " -np 4" +
-			     " ./test/openmpi"])
+    os.environ['PATH'] += ":" + os.path.dirname(OPENMPI_MPIRUN)
+  S=3*DEFAULT_S
+  runTest("openmpi", [5,6], [OPENMPI_MPIRUN + " -np 4" +
+                             " ./test/openmpi"])
   S=DEFAULT_S
   if oldPath:
     os.environ['PATH'] = oldPath
