@@ -12,9 +12,11 @@ int main(int argc, char* argv[])
   MPI_Comm_rank(MPI_COMM_WORLD, &rank);
   MPI_Comm_size(MPI_COMM_WORLD, &size);
 
+  if (rank == 0)
+    printf("*** Will print ten rows of dots.\n");  /* 1e7 / 1e6 == 10 */
   printf("Hello, world, I am %d of %d\n", rank, size);
 
-  for (i = 1; i < (int)1e8; i++)
+  for (i = 1; i < (int)1e7; i++)
   { int buf;
     MPI_Status status;
 
