@@ -6,7 +6,7 @@
 
 #include <stdio.h>
 #include <sys/time.h>
-#include "dmtcpplugin.h"
+#include "dmtcp.h"
 
 void print_time() {
   struct timeval val;
@@ -15,8 +15,6 @@ void print_time() {
 }
 
 unsigned int sleep(unsigned int seconds) {
-  static unsigned int (*next_fnc)() = NULL; /* Same type signature as sleep */
-
   printf("sleep1: "); print_time(); printf(" ... ");
   unsigned int result = NEXT_FNC(sleep)(seconds);
   print_time(); printf("\n");
@@ -24,15 +22,15 @@ unsigned int sleep(unsigned int seconds) {
   return result;
 }
 
-void dmtcp_process_event(DmtcpEvent_t event, void* data)
+void dmtcp_event_hook(DmtcpEvent_t event, DmtcpEventData_t *data)
 {
   /* NOTE:  See warning in plugin/README about calls to printf here. */
   switch (event) {
-  case DMTCP_EVENT_PRE_CHECKPOINT:
+  case DMTCP_EVENT_WRITE_CKPT:
     printf("\n*** The plugin %s is being called before checkpointing. ***\n",
 	   __FILE__);
     break;
-  case DMTCP_EVENT_POST_CHECKPOINT:
+  case DMTCP_EVENT_RESUME:
     printf("*** The plugin %s has now been checkpointed. ***\n", __FILE__);
     break;
   default:
@@ -40,5 +38,5 @@ void dmtcp_process_event(DmtcpEvent_t event, void* data)
   }
 
   /* Call this next line in order to pass DMTCP events to later plugins. */
-  NEXT_DMTCP_PROCESS_EVENT(event, data);
+  DMTCP_NEXT_EVENT_HOOK(event, data);
 }
