@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -8,8 +9,10 @@
 
 int main() {
   int pipefd[2];
-  if (pipe(pipefd) != 0)
-    return 1;
+  if (pipe(pipefd) == -1) {
+    perror("pipe");
+    exit(1);
+  }
   int fd = open("/dev/null", O_RDONLY);
   struct pollfd pfd[1];
   pfd[0].fd = pipefd[0];
@@ -21,5 +24,6 @@ int main() {
    */
   int rc = poll(pfd, 1, -1); /* -1 means infinite timeout */
   printf("ERROR:  rc = %d; errno = %d\n", rc, errno);
+  close(fd);
   return 1;
 }
